/*
 * Copyright (c) 2018, salesforce.com, inc.
 * All rights reserved.
 * SPDX-License-Identifier: MIT
 * For full license text, see the LICENSE file in the repo root or https://opensource.org/licenses/MIT
 */
import * as http from 'http';
import * as https from 'https';
import { URL } from 'node:url';
import { WebSocketServer } from 'ws';
import { LogLevel } from '../../types/index.js';
/**
 * Special mechanism to take internal users through an authentication flow.
 */
function requestHandler(req, res) {
    // the second arg could really be anything, we're just interested in searchParams
    // also req.url can never be undefined, it's just bad typing
    const url = new URL(req.url, 'http://localhost');
    const { headers } = req;
    // This is a mechanism in place only for internal usage, terminate all other http requests
    // If the origin is unknown or does not originate from a trusted domain or its looking for
    // something beyond a redirectUrl, terminate the connection
    if (!headers['origin'] ||
        !headers['origin'].endsWith('.wc.crm.dev:6101') ||
        url.pathname !== '/') {
        res.statusCode = 401;
        res.end('Terminating unauthorized http requests');
        return;
    }
    const redirectUrl = url.searchParams.get('redirectUrl');
    if (redirectUrl) {
        res.statusCode = 302;
        res.setHeader('Location', redirectUrl);
        return res.end();
    }
    res.writeHead(200, { 'Content-Type': 'text/html' });
    res.end(`Visit https://&lt;salesforce-url&gt;?0.aura.mode=devpreview`);
}
const TRUSTED_ORIGINS = ['\\.w[a-z]\\.crm\\.dev:6101$', '\\.lightning\\.force\\.com$'];
/**
 * Validate http request before upgrading it to WebSocket connection.
 */
function validateIncomingConnection(request, socket, cb) {
    const { headers } = request;
    const reqOrigin = headers['origin'];
    if (
    // Only upgrade requests that are coming in from the hmr-client
    // https://developer.mozilla.org/en-US/docs/Web/API/WebSockets_API/Writing_WebSocket_client_applications#protocols
    headers['sec-websocket-protocol'] !== 'lightning-dev-preview' ||
        !reqOrigin ||
        !TRUSTED_ORIGINS.some((trustedOrigin) => new RegExp(trustedOrigin).test(reqOrigin))) {
        socket.write('HTTP/1.1 401 Unauthorized\r\n\r\n');
        socket.destroy();
    }
    cb();
}
/**
 * Start a WebSocketServer.
 *
 * Note: Based on examples in the ws git repo: https://github.com/websockets/ws/blob/master/examples/express-session-parse/index.js
 * @param param0
 * @param logger
 * @returns
 */
export function startWebSocketServer({ port, https: httpsConfig }, logger) {
    try {
        // Start a WebSocketServer disconnected from the http server, allows us to handle authantication before upgrading ws connections
        const wss = new WebSocketServer({ noServer: true });
        const httpServer = httpsConfig ? https.createServer(httpsConfig) : http.createServer();
        httpServer.maxRequestsPerSocket = 1;
        // Special handler for the internal usecase
        httpServer.on('request', requestHandler);
        // Handle upgrade of connections after verifying the validity
        httpServer.on('upgrade', (request, socket, head) => {
            validateIncomingConnection(request, socket, () => {
                wss.handleUpgrade(request, socket, head, function (ws) {
                    wss.emit('connection', ws, request);
                });
            });
        });
        httpServer.listen(port, () => {
            logger.log(LogLevel.info, `[LWC Dev Server] LWC Dev Server started at port:${port}`);
        });
        // When the WebSocketServer is being closed, close the http server as well.
        wss.on('close', () => {
            httpServer.close();
        });
        return wss;
    }
    catch (e) {
        logger.log(LogLevel.error, `[LWC Dev Server] Failed to start a WebSocket server with the following error: ${e}`);
        process.exit(1);
    }
}
//# sourceMappingURL=web-socket-server.js.map